#pragma once

#include <QMainWindow>
#include "common/NCTypes.hxx"

namespace Ui
{
    class MainWindow;
}
class QSettings;
class QTranslator;
class NCFile;
class NCFilesystem;

namespace Language
{
    enum Type : quint8 {
        English,
        French,
        German
    };
}

namespace Mode
{
    enum Type : quint8 {
        Fresh,
        Edited,
        Selected
    };
}

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    // internal typedefs
    using ColType = NC::FileData::Type;

    // constructors and deconstructors
    explicit MainWindow(QWidget *parent = 0);
    virtual ~MainWindow();

public slots:
    // public slots
    void slotAppNew();
    void slotAppOpen();
    void slotAppSave();
    void slotAppSaveAs();
    void slotAppQuit();

    void slotFileAdd();
    void slotFileReplace();
    void slotFileRemove();
    void slotFileSavePlain();
    void slotFileSavePAK();
    void slotFileSaveZLIB();
    void slotFileConvWin();
    void slotFileConvLin();

    void slotLangEnglish();
    void slotLangFrench();
    void slotLangGerman();

    void slotStyleDefault();
    void slotStyleFusion();
    void slotStyleWindows();
    void slotStyleGTK();

    void slotAboutApp();
    void slotAboutQt();

    void slotStatusUpdate();

    void slotTableSelected();
    void slotTableChange(qint32 row);
    void slotTableStatus(quint32 from_row, quint32 to_row, ColType cols);
    void slotTableAdd(NCFile& entry, quint32 row);
    void slotTableReplace(NCFile& entry, quint32 row);
    void slotTableRemove(quint32 row);

protected:
    // protected stuff
    virtual void changeEvent(QEvent *event);
    void setLanguage(const Language::Type lang = Language::English);
    void setMode(const Mode::Type mode = Mode::Fresh);
    void setupActions();

private:
    Ui::MainWindow *_ui;
    QTranslator *_translator;
    QSettings *_settings;
    NCFilesystem *_vfs;
    QVector<QPair<qint32, qint32>> _marks;
    QString _filename;
};
