#pragma once

#include <QObject>
#include "common/NCTypes.hxx"

class NCFile : public QObject {
    Q_OBJECT
signals:
    void sigFileData(NCFile *file, quint8 type);

public:
    // internal typedefs
    enum : quint32 {
        Error = NC::Meta::Error,
        HSize = sizeof (quint32) * 5
    };

    // constructors and deconstructors
    //explicit NCFile();
    NCFile(QObject *parent = 0,
           const quint32& hsize = Error, const quint32& pos = Error, const quint32& csize = Error,
           const quint32& osize = Error, const quint32& nlen = Error, const QString& name = "",
           const QByteArray& cdata = "");
    NCFile(const NCFile& rhs);
    NCFile(const NCFile&& rhs);
    virtual ~NCFile();

    // operators
    NCFile& operator=(const NCFile& rhs);
    NCFile& operator=(const NCFile&& rhs);

    bool operator==(const NCFile& rhs) const;
    bool operator!=(const NCFile& rhs) const;

    // setters and getters
    void setHSize(const quint32& size = 0);
    quint32 hSize() const;

    void setPosition(const quint32& pos = 0);
    quint32 position() const;

    void setCSize(const quint32& size = 0);
    quint32 cSize() const;

    void setOSize(const quint32& size = 0);
    quint32 oSize() const;

    void setNameLength(const quint32& len = 0);
    quint32 nameLength() const;

    void setName(const QString& name = "");
    QString name() const;

    void setCData(const QByteArray& data = "");
    QByteArray cData() const;

    void setOData(const QByteArray& data = "");
    QByteArray oData() const;

    // status stuff
    quint32 size() const;
    quint32 headerSize() const;
    bool valid() const;

private:
    // data fields
    quint32 _hsize;
    quint32 _position;
    quint32 _csize;
    quint32 _osize;
    quint32 _name_length;
    QString _name;
    QByteArray _cdata;
};
